#lang racket/base
;;; saxo-tones.rkt
;;; from code in piano-tones.rkt and :
;;; http://theremin.music.uiowa.edu/MISaltosaxophone.html   (NoVib.mf)

(require rsound rsound/util ; rsound/draw
         racket/runtime-path
         (only-in racket/list first)
         (only-in racket/match match-define match))

(provide saxo-tone)

(define-runtime-path piano-samples "saxo-samples")

;; load some samples

(define C4samp (rs-read (build-path piano-samples "SaxoC4-clipped.wav"))) ; 60
(define G4samp (rs-read (build-path piano-samples "SaxoG4-clipped.wav"))) ; 67
(define G4bsamp (rs-read (build-path piano-samples "SaxoG4b-clipped.wav"))) ; 66
(define B4samp (rs-read (build-path piano-samples "SaxoB4-clipped.wav"))) ; 71
(define C5samp (rs-read (build-path piano-samples "SaxoC5-clipped.wav"))) ; 72
(define E5samp (rs-read (build-path piano-samples "SaxoE5-clipped.wav"))) ; 76

(define sample-table
  `((60 ,C4samp)
    (67 ,G4samp)
    (66 ,G4bsamp)
    (71 ,B4samp)
    (72 ,C5samp)
    (76 ,E5samp)))

;; find-nearest-sample : given a midi note number, find the 
;; note number and rsound of the sampled note with the
;; closest midi note number.
;; real -> (list/c real rsound)
;; NB: if this table were longer we might care about efficiency.
(define (find-nearest-sample_orig nn)
  (for/fold ([nearest (first sample-table)])
            ([pr (in-list sample-table)])
    (cond [(< (abs (- (first pr) nn)) (abs (- (first nearest) nn)))
           pr]
          [else nearest])))

(define (find-nearest-sample nn)
  (define (iter L sample dist)    ; old style
    (if (null? L) 
        sample
        (let* ((s (car L)) (d (abs (- nn (car s)))))
          (if (< d dist) 
              (iter (cdr L) s d)
              (iter (cdr L) sample dist)))))
  (iter (cdr sample-table) (car sample-table) (abs (- nn (caar sample-table)))))

(printf "Nearest sample of 70 is ~a\n" (find-nearest-sample 70))

(module+ test
  (require rackunit)
  (check-equal? (find-nearest-sample 70) (list 71 B4samp)))

;; produce a (resampled) note of a given frequency
(define (resampled-saxo f)
  (define desired-note-num (pitch->midi-note-num f))
  (match-define (list nn snd) (find-nearest-sample desired-note-num))
  (define best-pitch (midi-note-num->pitch nn))
  (resample/interp (/ f best-pitch) snd))

;; convert a list of lists of length 2 to a hash table
(define (assoc-list-to-hash l)
  (apply hasheq (apply append l)))

;; resample these to produce samples ... memoized.
(define resampled-saxo-table (make-hasheq))
(define (resampled-saxo/nn/memo nn)
  (match (hash-ref resampled-saxo-table nn #f)
    [#f 
     (define new-sound (resampled-saxo (midi-note-num->pitch nn)))
     (hash-set! resampled-saxo-table nn new-sound)
     new-sound]
    [other
     other]))

(define (saxo-tone nn)
  (resampled-saxo/nn/memo nn))



